//
// UI/Effect.fx
// Pass
//		0	:	Ring
//		1	:	Bar
//		2	:	Data
//		3	:	Repair Rate


/*****  O[oϐ  *****/
// {ϐ
float4x4	mWorld;								// [hs
float4x4	mView;								// r[s
float4x4	mProj;								// es
float3		LightDir;							// Cg
float3		EyePos;								// _ʒu
float4		Diffuse;

float4		Color;								// F

float2		InputTexel;
float		count;
float2		Offset;

float		RepairRate;							// C

/*****  eNX`  *****/
// C
texture Tex;
sampler TexSmp = sampler_state {
	Texture = <Tex>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

texture InputTex;
sampler InputTexSmp = sampler_state {
	Texture = <InputTex>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

texture NoiseTex;
sampler NoiseTexSmp = sampler_state {
	Texture = <NoiseTex>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};


/*****  \  *****/
// _VF[_sNZVF[_ɓn\

// ʏ
struct VS_OUT
{
	float4 Pos		: POSITION;		// 2DW
	float3 WorldPos	: TEXCOORD1;	// 3DW
	float2 TexUV	: TEXCOORD0;	// eNX`UV
	float4 Col		: COLOR0;		// _J[
	float3 Normal	: TEXCOORD2;	// @
	float3 Eye		: TEXCOORD3;
};


/*****  _VF[_  *****/

VS_OUT VSBasic(
	float4 pos		: POSITION,				// W
	float3 normal : NORMAL,				// @
	float2 texUV : TEXCOORD0				// eNX`UV
	)
{
	VS_OUT Out = (VS_OUT)0;

	Out.Pos = mul(pos, mWorld);
	Out.WorldPos = Out.Pos.xyz;
	Out.Pos = mul(Out.Pos, mView);
	Out.Pos = mul(Out.Pos, mProj);

	Out.TexUV = texUV;

	Out.Normal = normalize(mul(normal.xyz, (float3x3)mWorld));
	Out.Eye = normalize(EyePos - Out.Pos.xyz);

	return Out;
}

VS_OUT VSBack(
	float4 pos		: POSITION,				// W
	float3 normal : NORMAL,				// @
	float2 texUV : TEXCOORD0				// eNX`UV
	)
{
	VS_OUT Out = (VS_OUT)0;

	float4x4 scale = float4x4(
		1.1f, 0.0f, 0.0f, 0.0f,
		0.0f, 1.1f, 0.0f, 0.0f,
		0.0f, 0.0f, 1.1f, 0.0f,
		0.0f, 0.0f, 0.0f, 1.0f
		);

	//float4x4 backWorld = mul(scale, mWorld);

	Out.Pos = mul(pos, mWorld);
	Out.WorldPos = Out.Pos.xyz;
	Out.Pos = mul(Out.Pos, mView);
	Out.Pos = mul(Out.Pos, mProj);

	Out.TexUV = texUV;

	//Out.Normal = normalize(mul(normal.xyz, (float3x3)mWorld));
	Out.Normal = normalize(normal.xyz);

	return Out;
}


VS_OUT VSGauge(
	float4 pos		: POSITION,			// W
	float2 texUV : TEXCOORD0			// eNX`UV
	)
{
	VS_OUT Out = (VS_OUT)0;

	Out.Pos = mul(pos, mWorld);
	Out.WorldPos = Out.Pos.xyz;
	Out.Pos = mul(Out.Pos, mView);
	Out.Pos = mul(Out.Pos, mProj);

	Out.TexUV = texUV;

	Out.Normal = (float3)0;
	Out.Eye = (float3)0;
	
	return Out;
}

/*****  sNZVF[_  *****/

float4 PSNormal(VS_OUT In)	:	COLOR0
{
	In.TexUV += InputTexel * 0.5;
	float4 col = tex2D(InputTexSmp, In.TexUV);
	col *= Color;
	return col;
}

// C
float4 PSBase(VS_OUT In)	:	COLOR0
{
	float4 retCol = tex2D(TexSmp, In.TexUV);

	float p = dot(In.Normal, float3(0, 1, 0));
	//retCol += Color * (1 - abs(p));
	//retCol.a = 0.6;
	//retCol *= Color;
	retCol *= Diffuse;
	return retCol;
}

// wi
float4 PSBack(VS_OUT In)	:	COLOR0
{
	float4 retCol = Color;
	retCol.a = 1;
	return retCol;
}

float4 PSBackDeform(VS_OUT In)	:	COLOR0
{
	In.TexUV += InputTexel * 0.5;
	float4 col = tex2D(InputTexSmp, In.TexUV);
	//col *= 2.0f;

	float offset = (col.r + col.g + col.b) * 0.333333;

	In.TexUV.x -= offset * InputTexel.x * 30;
	col += tex2D(TexSmp, In.TexUV);

	return col;
}

float4 PSBlurX(VS_OUT In)	:	COLOR0
{
	float4 color = (float4)0;

	for (int i = 0; i < 10;i++) {
		color += tex2D(InputTexSmp, In.TexUV + float2(InputTexel.x * i, 0));
		color += tex2D(InputTexSmp, In.TexUV + float2(-InputTexel.x * i, 0));
	}

	color *= 1.0f / 20.0f;
	return color;
}

float4 PSBlurY(VS_OUT In)	:	COLOR0
{
	float4 color = (float4)0;

	for (int i = 0; i < 10; i++) {
		color += tex2D(InputTexSmp, In.TexUV + float2(0, InputTexel.y * i));
		color += tex2D(InputTexSmp, In.TexUV + float2(0, -InputTexel.y * i));
	}
	color *= 1.0f / 20.0f;
	return color;
}

float4 PSDataKey(VS_OUT In)	:	COLOR0
{
	// InputTexɃ}bvK

	float4 alphaMap = tex2D(InputTexSmp, float2(0, In.TexUV.y * 2.0f + count * InputTexel.y));

	// }bvRl傫قǂ炷
	In.TexUV += Offset;
	In.TexUV.x -= alphaMap.r * InputTexel.x * 10;
	float4 texCol = tex2D(TexSmp, In.TexUV);
	float4 noise = tex2D(NoiseTexSmp, In.TexUV + (count * InputTexel) * 40);

	float4 retCol = ((texCol * 0.8f) + (noise * 0.2f)) + Color * 0.6f;
	return retCol;
}

float4 PSGauge(VS_OUT In)	:	COLOR0
{
	clip(RepairRate - In.TexUV.x);
	return tex2D(TexSmp, In.TexUV);
}

/*****  eNjbN  *****/
technique TechBorder{
	pass Border {
		VertexShader = compile vs_3_0 VSBasic();
		PixelShader = compile ps_3_0 PSBase();
	}

	pass BlurX {
		PixelShader = compile ps_3_0 PSBlurX();
		ZEnable = false;
		ZWriteEnable = false;
	}

	pass BlurY {
		PixelShader = compile ps_3_0 PSBlurY();
		ZEnable = false;
		ZWriteEnable = false;
	}

	pass Normal {
		PixelShader = compile ps_3_0 PSNormal();
		ZEnable = false;
		ZWriteEnable = false;
	}
}

technique TechBack{
	pass Back {
		VertexShader = compile vs_3_0 VSBack();
		PixelShader = compile ps_3_0 PSBack();
	}

	pass BlurX {
		PixelShader = compile ps_3_0 PSBlurX();
		ZEnable = false;
		ZWriteEnable = false;
	}

	pass BlurY {
		PixelShader = compile ps_3_0 PSBlurY();
		ZEnable = false;
		ZWriteEnable = false;
	}

	pass BackDeform {
		PixelShader = compile ps_3_0 PSBackDeform();
		ZEnable = false;
		ZWriteEnable = false;
	}
}

technique TechDataKey {
	pass DataKey {
		VertexShader = compile vs_3_0 VSBasic();
		PixelShader = compile ps_3_0 PSDataKey();
		CullMode = CCW;
	}
}

technique TechGauge {
	pass DataKey {
		VertexShader = compile vs_3_0 VSGauge();
		PixelShader = compile ps_3_0 PSGauge();
		CullMode = CCW;
	}
}